﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Runtime.InteropServices;

namespace Info2Praktikum
{
    /// <summary>
    /// A class to get and set single pixels of a bitmap 
    /// </summary>
    /// <date>2011-05-03</date>
    public class Bitmap
    {
        int width;
        int height;
        byte[] bytes;

        /// <summary>
        /// Build a black bitmap
        /// </summary>
        /// <param name="width">width</param>
        /// <param name="height">height</param>
        public Bitmap(int width, int height)
        {
            this.width = width;
            this.height = height;
            bytes = new byte[4 * width * height];
            for (int i = 0; i < width*height; i++)
            {
                bytes[4 * i + 3] = 255; // make opaque
            }
        }

        /// <summary>
        /// Build a bitmap from a given file
        /// </summary>
        /// <param name="path">file name</param>
        public Bitmap(string path)
        {
            BitmapSource b = new BitmapImage(new Uri(path));
            if (b.Format != PixelFormats.Bgra32)
            {
                b = new FormatConvertedBitmap(b, PixelFormats.Bgra32, null, 0);
            }
            width = b.PixelWidth;
            height = b.PixelHeight;
            bytes = new byte[4 * width * height];
            b.CopyPixels(bytes, 4 * width, 0);
        }

        /// <summary>
        /// The bitmap's width
        /// </summary>
        public int Width
        {
            get
            {
                return width;
            }
        }

        /// <summary>
        /// The bitmap's height
        /// </summary>
        public int Height
        {
            get
            {
                return height;
            }
        }

        /// <summary>
        /// Get the value of the blue channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <returns>value</returns>
        public byte GetBlue(int x, int y)
        {
            return bytes[4 * (width * y + x)];
        }

        /// <summary>
        /// Get the value of the green channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <returns>value</returns>
        public byte GetGreen(int x, int y)
        {
            return bytes[4 * (width * y + x) + 1];
        }

        /// <summary>
        /// Get the value of the blue channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <returns>value</returns>
        public byte GetRed(int x, int y)
        {
            return bytes[4 * (width * y + x) + 2];
        }

        /// <summary>
        /// Set the value of the red channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <param name="c">value</param>
        public void SetBlue(int x, int y, int c)
        {
            bytes[4 * (width * y + x)] = (byte)c;
        }

        /// <summary>
        /// Set the value of the green channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <param name="c">value</param>
        public void SetGreen(int x, int y, int c)
        {
            bytes[4 * (width * y + x) + 1] = (byte)c;
        }

        /// <summary>
        /// Set the value of the blue channel of the pixel at (x, y)
        /// </summary>
        /// <param name="x">x</param>
        /// <param name="y">y</param>
        /// <param name="c">value</param>
        public void SetRed(int x, int y, int c)
        {
            bytes[4 * (width * y + x) + 2] = (byte)c;
        }

        /// <summary>
        /// Builds an ImageSource to be used e.g. in an Image control
        /// </summary>
        /// <returns>ImageSource</returns>
        public System.Windows.Media.ImageSource GetImage()
        {
            WriteableBitmap w = new WriteableBitmap(width, height, 96.0, 96.0, PixelFormats.Bgra32, null);
            w.WritePixels(new Int32Rect(0, 0, width, height), bytes, 4*width, 0);
            return w;
        }

        public void WriteAsText(string path)
        {
            System.IO.FileStream fs = new System.IO.FileStream(path, System.IO.FileMode.Create);
            System.IO.StreamWriter sw = new System.IO.StreamWriter(fs);
            sw.WriteLine(Width);
            sw.WriteLine(Height);
            for (int y = 0; y < Height; y++)
            {
                for (int x = 0; x < Width; x++)
                {
                    sw.Write(GetRed(x, y));
                    sw.Write(' ');
                    sw.Write(GetGreen(x, y));
                    sw.Write(' ');
                    sw.Write(GetBlue(x, y));
                    sw.Write(' ');
                }
                sw.WriteLine();
            }
            sw.Close(); // unschön: wird bei Exception übersprungen. Besser mit using oder finally.
        }

        static public Bitmap ReadFromText(string path)
        {
            System.IO.FileStream fs = new System.IO.FileStream(path, System.IO.FileMode.Open);
            System.IO.StreamReader sr = new System.IO.StreamReader(fs);
            int w = int.Parse(sr.ReadLine());
            int h = int.Parse(sr.ReadLine());
            Bitmap bmp = new Bitmap(w, h);
            for (int y = 0; y < h; y++)
            {
                string line = sr.ReadLine();
                string[] parts = line.Split(' ');
                for (int x = 0; x < w; x++)
                {
                    bmp.SetRed(x, y, int.Parse(parts[3*x]));
                    bmp.SetGreen(x, y, int.Parse(parts[3*x+1]));
                    bmp.SetBlue(x, y, int.Parse(parts[3*x+2]));
                }
            }
            sr.Close(); // unschön: wird bei Exception übersprungen. Besser mit using oder finally.
            return bmp;
        }
    }
}